@@ -1,15 +1,15 @@
 # -*- coding: utf-8 -*-
 
-import audioinfo, os, pdb, sys, string, re
-from decimal import Decimal, InvalidOperation
-from pyparsing import (Word, alphas,Literal, OneOrMore, NotAny, alphanums, 
-    nums, ZeroOrMore, Forward, delimitedList, Combine, QuotedString, 
-    CharsNotIn, White, originalTextFor, nestedExpr, 
-    Optional, commaSeparatedList)
-from puddleobjects import PuddleConfig
+import pdb
+import audioinfo, os, string, re
+from decimal import Decimal
+from pyparsing import (Word, alphas,Literal, OneOrMore, alphanums, 
+    nums, delimitedList, Combine, QuotedString, 
+    CharsNotIn, originalTextFor, nestedExpr, 
+    Optional)
+from puddleobjects import PuddleConfig, safe_name
 from funcprint import pprint
-from puddlestuff.util import PluginFunction, translate
-numtimes = 0 #Used in filenametotag to keep track of shit.
+from puddlestuff.util import PluginFunction, translate, to_list, to_string
 import cPickle as pickle
 stringtags = audioinfo.stringtags
 from copy import deepcopy
@@ -17,6 +17,8 @@ from constants import ACTIONDIR, CHECKBOX, SPINBOX, SYNTAX_ERROR, SYNTAX_ARG_ERR
 import glob
 from collections import defaultdict
 from functools import partial
+numtimes = 0
+import string
 
 NOT_ALL = audioinfo.INFOTAGS + ['__image']
 FILETAGS = audioinfo.FILETAGS
@@ -24,6 +26,9 @@ FUNC_NAME = 'func_name'
 FIELDS = 'fields'
 FUNC_MODULE = 'module'
 ARGS = 'arguments'
+KEYWORD_ARGS = set(['tags', 'm_tags', 'r_tags', 'state'])
+
+whitespace = set(unicode(string.whitespace))
 
 class ParseError(Exception):
     def __init__(self, message):
@@ -32,12 +37,15 @@ class ParseError(Exception):
 
 class FuncError(ParseError): pass
 
-from functions import functions, no_fields
+class MultiValueError(FuncError): pass
+
+from functions import functions
 
 def arglen_error(e, passed, function, to_raise = True):
     varnames = function.func_code.co_varnames[:function.func_code.co_argcount]
     args_len = len(passed)
     param_len = len(varnames)
+    message = None
     if args_len > param_len:
         message = translate('Functions',
             'At most %1 arguments expected. %2 given.')
@@ -49,7 +57,10 @@ def arglen_error(e, passed, function, to_raise = True):
             'At least %1 arguments expected. %2 given.')
     else:
         raise e
-    message = message.arg(unicode(param_len)).arg(unicode(args_len))
+    if message is not None:
+        message = message.arg(unicode(param_len)).arg(unicode(args_len))
+    else:
+        raise e
     if to_raise:
         raise ParseError(message)
     else:
@@ -66,7 +77,7 @@ def convert_actions(dirpath, new_dir):
     for filename in glob.glob(path_join(dirpath, '*.action')):
         funcs, name = get_old_action(filename)
         os.rename(filename, path_join(backup, basename(filename)))
-        save_action(path_join(new_dir, basename(filename)), name, funcs)
+        save_macro(path_join(new_dir, basename(filename)), name, funcs)
 
 def filenametotag(pattern, filename, checkext = False, split_dirs=True):
     """Retrieves tag values from your filename
@@ -137,7 +148,7 @@ def get_old_action(filename):
     f.close()
     return [funcs, name]
 
-def load_action(filename):
+def load_macro_info(filename):
     modules = defaultdict(lambda: defaultdict(lambda: {}))
     for function in functions.values():
         if isinstance(function, PluginFunction):
@@ -148,7 +159,14 @@ def load_action(filename):
     cparser = PuddleConfig(filename)
     funcs = []
     name = cparser.get('info', 'name', u'')
-    for section in cparser.sections():
+
+    def get_func_index(funcname):
+        if funcname.startswith('Func'):
+            return int(funcname[4:].strip())
+        else:
+            return funcname
+
+    for section in sorted(cparser.sections(), key=get_func_index):
         if section.startswith(u'Func'):
             get = partial(cparser.get, section)
             func_name = get(FUNC_NAME, u'')
@@ -159,10 +177,12 @@ def load_action(filename):
                 func = Function(modules[func_module][func_name], fields)
             except IndexError:
                 continue
+            except AttributeError:
+                continue
             newargs = []
             for i, (control, arg) in enumerate(zip(func.controls, arguments)):
                 if control == CHECKBOX:
-                    if arg == u'False':
+                    if arg == False or arg == u'False':
                         newargs.append(False)
                     else:
                         newargs.append(True)
@@ -175,23 +195,11 @@ def load_action(filename):
             else:
                 func.args = arguments
             funcs.append(func)
-    return [funcs, name]
+    return [funcs, name]    
 
-def load_action_from_name(name):
+def load_macro_from_name(name):
     filename = os.path.join(ACTIONDIR, safe_name(name) + '.action')
-    return get_action(filename)
-
-def getfunc(text, audio, dictionary=None):
-    """Parses text and replaces all functions
-    with their appropriate values.
-
-    Function must be of the form $name(args)
-    Returns the text unmodified if unsuccesful,"""
-
-    if not isinstance(text, unicode):
-        text = unicode(text, 'utf8')
-
-    return function_parser(audio, dictionary=dictionary).transformString(text)
+    return Macro(filename)
 
 def func_tokens(dictionary, parse_action):
     func_name = Word(alphas+'_', alphanums+'_')
@@ -221,95 +229,239 @@ def func_tokens(dictionary, parse_action):
 
     return func_tok, arglist, rx_tok
 
-def parse_arglist(text):
-    in_quote=False
-    escape = False
-    current = []
-    arglist = []
-    for i, c in enumerate(text):
-        if c == u',' and not in_quote:
-            arglist.append(u''.join(current).strip())
-            current = []
-            continue
-        elif c == u'"' and not escape:
-            in_quote = not in_quote
-        elif c == u'\\':
-            escape = not escape
-        current.append(c)
+def get_function_arguments(funcname, func, arguments, reserved, fmt=True, *dicts):
 
-    if current:
-        arglist.append(u''.join(current).strip())
-    elif text.endswith(u','):
-        arglist.append(u'')
+    varnames = func.func_code.co_varnames[:func.func_code.co_argcount]
 
-    return [z if z else u'""' for z in arglist]
+    #arguments will contain only a list of user supplied arguments
+    #Eg. for the function $format(%artist%) the user will specify
+    #only the %artist%, whereas calling the function requires
+    #the tags in order to look it up.
+    #This, and below replaces the tags with their corresponding order.
+    
+    topass = {}
+    othervars = []
 
-def function_parser(m_audio, audio=None, dictionary=None):
-    """Parses a function in the form $name(arguments)
-    the function $name from the functions module is called
-    with the arguments."""
-    def replaceNestedMacros(tokens):
-        #if tokens.funcname == 'if':
-            #pdb.set_trace()
-        if func_tok.searchString(tokens.args):
-            tokens['args'] = func_tok.transformString(tokens.args)
+    key_args = set(KEYWORD_ARGS).union(reserved)
+    
+    for i, v in enumerate(varnames):
+        if v in key_args:
+            topass[v] = reserved[v]
+        else:
+            othervars.append(v)
 
-        if tokens.funcname not in functions:
-            return u''
-        function = functions[tokens.funcname]
-        
-        if tokens.args == u'()':
+    for no, (arg, param) in enumerate(zip(arguments, othervars)):
+        if param.startswith('p_'):
+            topass[param] = arg
+        elif param.startswith('n_'):
             try:
-                return function()
-            except TypeError, e:
-                arglen_error(e, [], function)
-
-        arguments = parse_arglist(tokens.args[1:-1])
-        arguments = arglist.parseString(u','.join(arguments)).asList()
-
-        varnames = function.func_code.co_varnames
-        
-        for i,v in enumerate(varnames):
-            if v == 'tags':
-                arguments.insert(i, audio)
-            elif v == 'm_tags':
-                arguments.insert(i, m_audio)
-        topass = []
-        for no, (arg, param) in enumerate(zip(arguments, varnames)):
-            if param.startswith('t_') or param.startswith('text'):
-                topass.append(replacevars(arg, audio, dictionary))
-            elif param.startswith('n_'):
-                try:
-                    if float(arg) == int(arg):
-                        topass.append(int(arg))
-                    else:
-                        topass.append(float(arg))
-                except ValueError:
-                    raise ParseError(SYNTAX_ARG_ERROR % (tokens.funcname, no))
+                if float(arg) or float(arg) == 0:
+                    topass[param] = Decimal(arg)
+            except ValueError:
+                raise ParseError(SYNTAX_ARG_ERROR % (funcname, no + 1))
+        else:
+            if isinstance(arg, basestring) and fmt:
+                topass[param] = replacevars(arg, *dicts)
             else:
-                topass.append(arg)
+                topass[param] = arg
+
+    return topass
+
+def run_format_func(funcname, arguments, m_audio, s_audio=None, extra=None,
+    state=None):
+    '''Runs the function function using the arguments specified from pudlestuff.function.
+
+    Arguments:
+    funcname  -- String with the function name. Looked up using the
+                 dictionary pudlestuff.function.functions
+    arguments -- List of arguments to pass to the function. Patterns
+                 should not be evaluated. They'll be evaluated here.
+    m_audio   -- Audio file containg multiple multiple values per key.
+                 Eg. {'artist': [u'Artist1': 'Artist2']}
+
+    Keyword Arguments
+    s_audio -- Same as m_audio, but containing strings as values.
+               Generated on each run unless also passed.
+    extra -- Dictionary containing extra fields that are to be used
+             when matching fields.
+    state -- Dictionary that hold state. Like {'__count': 15}.
+             Used by some functions in puddlestuff.functions
+    
+    '''
+    
+    #Get function
+    try:
+        if isinstance(funcname, basestring):
+            func = functions[funcname]
+        else:
+            func = funcname
+    except KeyError:
+        raise ParseError(SYNTAX_ERROR.arg(funcname).arg(
+            translate('Defaults', 'function does not exist.')))
+
+    extra = {} if extra is None else extra
+    s_audio = stringtags(m_audio) if s_audio is None else s_audio
+    
+    reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state}
+    dicts = [s_audio, extra, state]
+    topass = get_function_arguments(funcname, func, arguments, 
+        reserved, True, *dicts)
+
+    try:
+        ret = func(**topass)
+        if ret is None:
+            return u''
+        return ret
+    except TypeError, e:
+        message = SYNTAX_ERROR.arg(funcname)
+        message = message.arg(arglen_error(e, topass, func, False))
+        raise ParseError(message)
+    except FuncError, e:
+        message = SYNTAX_ERROR.arg(funcname).arg(e.message)
+        raise ParseError(message)
+
+def parsefunc(s, m_audio, s_audio=None, state=None, extra=None, ret_i=False, path_sep=None):
+    """Parses format strings. Returns the parsed string.
+
+    Arguments
+    ---------
+    s  -- *Unicode* format string. Eg. $replace(%artist%, name, surname)
+    m_audio -- Audio file containg multiple multiple values per key.
+        Eg. {'artist': [u'Artist1': 'Artist2']}
+
+    Keyword Arguments
+    s_audio -- Same as m_audio, but containing strings as values.
+        Generated on each run unless also passed.
+    extra -- Dictionary containing extra fields that are to be used
+             when matching fields.
+    state -- Dictionary that hold state. Like {'__count': 15}.
+             Used by some functions in puddlestuff.functions
+
+    >>> audio = {'artist': [u'Artist1'], 'track':u'10'}
+    >>> parsefunc(u'%track% - %artist%', audio)
+    Artist1 - 10
+    >>> state = {'__count': u'5'}
+    >>> parsefunc(u'$num(%track%, 2)/$num(%__count%, 2). %artist%', audio,
+    ... state = state)
+    u'05/05. Artist1'
+
+    """
+
+    #Yes I know this is a big ass function...but it works and I can't
+    #see a way to split it without making it complicated.
+    
+    tokens = []
+    token = []
+    #List containing a function with it's arguments
+    #Will look like ['replace', arg1, arg2, arg3]
+    #functions get evaluated as soon as closing bracket found.
+    func = []
+    #Flag determining if current within a function. Used for making comma's
+    #significant
+    in_func = False
+    in_quote = False
+    #field_open == -1 if not current in the middle of processing a field
+    #eg. %artist%. Otherwise contains the index in s that the field started.
+    field_open = -1
+    #Determine if next char should be escaped.
+    escape = False
+        
+    
+    s_audio = stringtags(m_audio) if s_audio is None else s_audio
+    state = {} if state is None else state
+    tags = s_audio.copy()
+    tags.update(state)
+    tags.update(extra if extra is not None else {})
+    escape_chars = set('()$%\\')
+
+    br_error = translate('Errors', 'No closing bracket found.')
+
+    paths = []
+
+    i = 0
+    while 1:
         try:
-            ret = function(*topass)
-            if ret is None:
-                return u''
-            return ret
-        except TypeError, e:
-            message = SYNTAX_ERROR.arg(tokens.funcname)
-            message = message.arg(arglen_error(e, topass, function, False))
-            raise ParseError(message)
-        except FuncError, e:
-            message = SYNTAX_ERROR.arg(tokens.funcname).arg(e.message)
-            raise ParseError(message)
+            c = s[i]
+        except IndexError:  #  Parsing's done.
+            if in_func:
+                raise ParseError(SYNTAX_ERROR.arg(func[0]).arg(br_error))
+            if token:
+                tokens.append(replacevars(u''.join(token), tags))
+            break
 
-    if audio is None:
-        audio = stringtags(m_audio)
+        if c == u'"' and not escape:
+            if in_func:
+                token.append(c)
+            in_quote = not in_quote
+        elif c == u'"' and escape and in_func:
+            token.append(u'\\"')
+            i += 1
+            escape = False
+            continue
+        elif escape:
+            token.append(c)
+            escape = False
+            i += 1
+            continue
+        elif in_quote:
+            token.append(c)
+        elif c == u'\\' and not escape:
+            escape = True
+            i += 1
+            try:
+                next_char = s[i]
+            except IndexError:
+                next_char = None
 
-    func_tok, arglist, rx_tok = func_tokens(dictionary, replaceNestedMacros)
+            if not in_func or (next_char not in escape_chars):
+                token.append(c)
+                escape = False
+            continue
+        elif c == u'$' and not (escape or (field_open >= 0)):
+            func_name = re.search(u'^\$(\w+)\(', s[i:])
+            if not func_name:
+                token.append(c)
+                i += 1
+                continue
 
-    return func_tok
+            if in_func:
+                func_parsed, offset = parsefunc(s[i:], m_audio, s_audio, state, extra, True)
+                token.append(func_parsed)
+                i += offset + 1
+                continue
 
-def parsefunc(text, audio, d=None):
-    return function_parser(audio, None, d).transformString(text)
+            tokens.append(replacevars(u''.join(token), tags))
+            token = []
+            func = []
+            func_name = func_name.groups(0)[0]
+            func.append(func_name)
+            in_func = True
+            i += len(func_name) + 1
+        elif in_func and not in_quote and not token and c in whitespace:
+            'just increment counter'
+        elif c == u',' and in_func and not in_quote:
+            func.append(u''.join(token))
+            token = []
+        elif c == u')' and in_func:
+            in_func = False
+            if token or s[i-1] == u',':
+                func.append(u''.join(token))
+            func_parsed = run_format_func(func[0], func[1:], m_audio, s_audio)
+            if ret_i:
+                return func_parsed, i
+            tokens.append(func_parsed)
+            token = []
+        else:
+            token.append(c)
+            if path_sep and c == path_sep and not in_func:
+                paths.append(len(u''.join(tokens) + replacevars(u''.join(token), tags)) - 1)
+        escape = False
+        i += 1
+
+    if path_sep and not ret_i:
+        return paths, u''.join(tokens)
+    else:
+        return u''.join(tokens)
 
 def parse_field_list(fields, audio, selected=None):
     fields = fields[::]
@@ -357,41 +509,71 @@ def removeSpaces(text):
         text = text.replace(char, '')
     return text.lower()
 
-_varpat = re.compile('%([\w ]+)%')
-def replacevars(text, dictionary, extra = None):
-    """Replaces the tags in pattern with their corresponding value in audio.
+def replacevars(pattern, *dicts):
+    """Replaces occurrences of %key% with the d[key] in the string pattern.
 
-    A tag is a string enclosed by percentages, .e.g. %tag%.
+    Arguments
+    ---------
+    pattern -- Format string like '%artist% - %title%'
+    d -- Dictionary with string values.
 
-    >>>replacevars('%artist% - %title%', {'artist':'Artist', 'title':'Title"}
-    Artist - Title."""
+    Optional Arguments
+    ------------------
+    *extra - Extra dictionaries to check. If a key can't be found in
+            d, the other dictionaries will be checked in order.
 
-    if extra is None:
-        extra = {}
+    Returns the parsed string. If a key isn't found, but in the format
+    string, it'll be removed.
+
+    >>> replacevars('%artist%', {'artist': u'ARTIST'})
+    u'ARTIST'
+    >>> replacevars('one %two%', {"three": u'VALUE'})
+    u'one '
+
+    """
+    r_vars = {}
+    map(r_vars.update, [z for z in dicts if z])
     
-    dictionary = stringtags(dictionary)
-    start = 0
-    l = []
-    append = l.append
-    for match in _varpat.finditer(text):
-        append(text[start: match.start(0)])
-        try: append(dictionary[match.groups()[0]])
-        except KeyError:
-            try: append(extra[match.groups()[0]])
-            except KeyError: pass
-        start = match.end(0)
-    else:
-        append(text[start:])
-    return u''.join(l)
+    in_quote = False
+    in_field = False
+    ret = []
+    field_start = 0
+    escape = False
+
+    for i, c in enumerate(pattern):
+        try:
+            next_char = pattern[i + 1]
+        except IndexError:
+            next_char = None
+        if c == u'\\' and next_char == '"' and not escape:
+            escape = True
+            continue
+        elif escape:
+            escape = False
+        elif c == u'"':
+            in_quote = not in_quote
+            continue
+        elif c == u'%' and not in_quote:
+
+            if not in_field:
+                field_start = len(ret)
+                in_field = True
+            elif in_field:
+                in_field = False
+                field = u''.join(ret[field_start:])
+                del(ret[field_start:])
+                ret.append(r_vars.get(field, u''))
+            continue
+        ret.append(c)
+
+    return u''.join(ret)
 
 
-def runAction(funcs, audio, state = None, quick_action=None):
+def apply_actions(actions, audio, state=None, ovr_fields=None):
     """Runs an action on audio
 
-    funcs can be a list of Function objects or a filename of an action file (see getAction).
+    actions can be a list of Function objects or a filename of an action file (see load_action).
     audio must dictionary-like object."""
-    if isinstance(funcs, basestring):
-        funcs = getAction(funcs)[0]
     
     if state is None:
         state = {}
@@ -405,13 +587,13 @@ def runAction(funcs, audio, state = None, quick_action=None):
         audio = deepcopy(audio.tags)
     else:
         audio = deepcopy(audio)
-    
+
     changed = set()
-    for func in funcs:
-        if quick_action is None:
+    for func in actions:
+        if ovr_fields is None:
             fields = parse_field_list(func.tag, audio)
         else:
-            fields = quick_action[::]
+            fields = ovr_fields[::]
         ret = {}
 
         for field in fields:
@@ -442,12 +624,23 @@ def runAction(funcs, audio, state = None, quick_action=None):
             audio.update(ret)
     return dict([(z, audio[z]) for z in changed])
 
+def apply_macros(macros, audio, state, fields=None):
+    actions = []
+    for m in macros:
+        actions.extend(m.actions)
+
+    if fields:
+        return apply_actions(actions, audio, state, fields)
+    else:
+        return apply_actions(actions, audio, state)
+    
+
 def runQuickAction(funcs, audio, state, tag):
     """Same as runAction, except that all funcs are 
     applied not in the values stored but on audio[tag]."""
-    return runAction(funcs, audio, state, tag)
+    return apply_macros(funcs, audio, state, tag)
 
-def save_action(filename, name, funcs):
+def save_macro(filename, name, funcs):
     f = open(filename, 'w')
     f.close()
     cparser = PuddleConfig(filename)
@@ -524,11 +717,11 @@ def tagtofilename(pattern, filename, addext=False, extension=None, state=None):
             return 0
 
     if not addext:
-        return replacevars(getfunc(pattern, tags, state), tags, state)
+        return parsefunc(pattern, tags, state=state)
     elif addext and (extension is not None):
-        return replacevars(getfunc(pattern, tags, state), tags, state) + os.path.extsep + extension
+        return parsefunc(pattern, tags, state=state) + os.path.extsep + extension
     else:
-        return replacevars(getfunc(pattern, tags, state), tags, state) + os.path.extsep + tags["__ext"]
+        return parsefunc(pattern, tags, state=state) + os.path.extsep + tags["__ext"]
 
 def tagtotag(pattern, text, expression):
     """See filenametotag for an implementation example and explanation.
@@ -621,50 +814,58 @@ class Function:
     def setArgs(self, args):
         self.args = args
 
-    def runFunction (self, text=None, m_tags=None, state=None, tags=None, r_tags=None):
-        function = self.function
-        varnames = function.func_code.co_varnames[:function.func_code.co_argcount]
-        
+    def runFunction (self, text=None, m_tags=None, state=None,
+        tags=None, r_tags=None):
+
+        func = self.function
+
+        varnames = func.func_code.co_varnames[:func.func_code.co_argcount]
+
         if not varnames:
-            return function()
-        
-        if tags is None:
-            tags = stringtags(m_tags)
-        
-        if state is None:
-            state = {}
-        
-        arguments = self.args[::]
-        first_text = True
+            return func()
+            
+        s_audio = stringtags(m_tags) if tags is None else tags
+        m_audio = m_tags
+        state = {} if state is None else state
 
-        d = {'tags': tags, 'r_tags': r_tags, 'm_tags': m_tags, 'state':state}
+        m_text = to_list(text)
+        text = to_string(text)
 
-        offset = 0
-        for i, v in enumerate(varnames):
-            if v in d:
-                arguments.insert(i + offset, d[v])
-                offset += 1
-
-        if varnames[0] in d:
-            first_text = False
+        reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
+            'r_tags': r_tags, 'm_text': m_text}
         
-        if first_text:
-            if isinstance(text, basestring):
-                if varnames[0].startswith('m_'):
-                    return function([text], *arguments)
-                else:
-                    return function(text, *arguments)
-            elif varnames[0].startswith('m_'):
-                return function(text, *arguments)
-            else:
-                ret = (function(v, *arguments) for v in text)
-                temp = []
-                append = temp.append
-                [append(z) for z in ret if z not in temp]
-                return temp
+        if varnames[0] in reserved:
+            reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
+                'r_tags': r_tags, 'text': to_string(text),
+                'm_text': m_text}
+            topass = get_function_arguments("", func, self.args, reserved,
+                False, *[s_audio, state])
+            return func(**topass)
         else:
-            return function(*arguments)
+            reserved = {'tags': s_audio, 'm_tags': m_audio, 'state': state,
+                'r_tags': r_tags}
+            topass = get_function_arguments("", func,
+                [text] + self.args, reserved, False, *[s_audio, state])
+            
+        try:
+            first_arg = [z for z in varnames if z not in reserved][0]
+        except IndexError:
+            return
 
+        if not first_arg.startswith('m_'):
+            text = [to_string(z) for z in to_list(m_text)]
+            ret = []
+            for z in m_text:
+                topass[first_arg] = z
+                ret.append(func(**topass))
+
+            temp = []
+            append = temp.append
+            [append(z) for z in ret if z not in temp]
+            return temp
+        else:
+            topass[first_arg]
+            return func(**topass)
 
     def description(self):
         d = [u", ".join(self.tag)] + self.args
@@ -691,3 +892,32 @@ class Function:
         if self.function.func_code.co_argcount > len(self.args) + 1:
             self.args.append(arg)
 
+class Macro(object):
+    def __init__(self, filename=None):
+        object.__init__(self)
+        self.name = u''
+        self.actions = []
+        self.filename = u''
+        if filename is not None:
+            self.load(filename)
+
+    def apply_action(self, audio, state=None, fields=None):
+
+        return apply_macros(self.actions, audio, state, fields)
+
+    def copy(self):
+        m = Macro()
+        m.filename = self.filename
+        m.actions = self.actions[::]
+        return m
+
+    def load(self, filename):
+        self.actions, self.name = load_macro_info(filename)
+        self.filename = filename
+
+    def save(self, filename=None):
+        if filename is None:
+            filename = self.filename
+    
+        save_macro(filename, self.name, self.actions)
+        
\ No newline at end of file
