@@ -37,6 +37,8 @@ import math
 import decimal, os, pdb, re, shutil, sys
 import string, time, traceback, unicodedata
 
+from collections import defaultdict
+
 from copy import deepcopy
 from functools import partial
 from operator import itemgetter
@@ -73,22 +75,27 @@ def autonumbering(r_tags, minimum=1, restart=False, padding=1, state=None):
 oi,spinbox,1
 aoeu,check, False
 au,spinbox,1'''
+
     if 'autonumbering' not in state:
-        state['autonumbering'] = {}
-    
-    dir_list = state['autonumbering']
-    
-    if restart:
-        key = r_tags.dirpath
-    else:
-        key = 'temp'
+        dircount = defaultdict(lambda: 0)
+        count = {}
+        
+        files = state.get('__files')
+        if files is None:
+            count = {1:u'1'}
+        else:
+            for i, tag in enumerate(state['__files'], 1):
+                if restart:
+                    dircount[tag.dirpath] += 1
+                    count[i] = unicode(dircount[tag.dirpath])
+                else:
+                    count[i] = unicode(i)
 
-    if key in dir_list:
-        dir_list[key] += 1
-    else:
-        dir_list[key] = minimum
+        state['autonumbering'] = count
+        
+    index = int(state.get('__counter', 1))
 
-    tracknum = unicode(dir_list[key])
+    tracknum = state['autonumbering'][index]
 
     if padding > 1:
         return _pad(tracknum, padding)
@@ -96,6 +103,8 @@ au,spinbox,1'''
         return tracknum
 
 def check_truth(text):
+    if isinstance(text, basestring):
+        text = text.strip()
     return 0 if ((not text) or (text == u'0')) else 1
 
 def and_(text, text1):
@@ -221,6 +230,10 @@ def grtr(text, text1):
     else:
         return false
 
+def to_num(text):
+    match = re.search('[\-\+]?[0-9]+(\.[0-9]+)?', text)
+    return match.group() if match else u''
+
 def hasformat(p_pat, tagname = "__filename"):
     if findfunc.filenametotag(p_pat, tagname):
         return true
@@ -643,7 +656,7 @@ class RegHelper(object):
         except KeyError:
             return u'""'
 
-def replaceWithReg(m_tags, text, regex, repl, matchcase=False, state=None):
+def replaceWithReg(m_tags, text, regex, repl, m_text=None, matchcase=False, state=None):
     """Replace with RegExp, "RegReplace $0: RegExp '$1' with '$2', Match Case: $3"
 &Regular Expression, text
 Replace &matches with:, text
@@ -652,6 +665,9 @@ Match &Case, check"""
     if not regex:
         return text
 
+    if m_text is None:
+        m_text = [text]
+
     if not check_truth(matchcase):
         flags = re.UNICODE | re.I
     else:
@@ -663,27 +679,28 @@ Match &Case, check"""
         if not group:
             d = {}
         elif groups:
-            d = dict((i + 1, g) for i, g in enumerate(groups))
+            d = dict([(i, z if z is not None else u"") for i, z in enumerate(groups, 1)])
             d[0] = group
         else:
             d = {1: group, 0: group}
 
-
         ret = re.sub('(?i)\$\d+', RegHelper(d).repl, repl, 0)
-        
         return findfunc.parsefunc(ret, m_tags)
 
-    try:
+    def replace_matches(value):
         try:
-            return re.sub(regex, replace_tokens, text, 0, flags)
-        except TypeError:
-            #Python2.6 doesn't accept flags arg.
-            if matchcase:
-                return re.sub(u'(?i)' + regex, replace_tokens, text, 0)
-            else:
-                return re.sub(regex, replace_tokens, text, 0)
-    except re.error, e:
-        raise findfunc.FuncError(unicode(e))
+            try:
+                return re.sub(regex, replace_tokens, value, 0, flags)
+            except TypeError:
+                #Python2.6 doesn't accept flags arg.
+                if matchcase:
+                    return re.sub(u'(?i)' + regex, replace_tokens, value, 0)
+                else:
+                    return re.sub(regex, replace_tokens, value, 0)
+        except re.error, e:
+            raise findfunc.FuncError(unicode(e))
+
+    return u"\\".join(replace_matches(z) for z in m_text)
 
 replace_regex = replaceWithReg
 
@@ -739,11 +756,9 @@ def rg2sc(gain, peak=None):
         if len(gain) == 2: #gain:peak
             peak = float(gain[1])
             gain = float(gain[0])
-            print peak, gain
         elif len(gain) == 3: #channel:gain:peak
             peak = float(gain[2])
             gain = float(gain[1])
-            print gain, peak
         else:
             return
     else:
@@ -1028,7 +1043,9 @@ functions = {
     "upper": upper,
     'update_from_tag': update_from_tag,
     "validate": validate,
-    'to_ascii': to_ascii}
+    'to_ascii': to_ascii,
+    'to_num': to_num
+}
 
 no_fields = (filenametotag, load_images, move, remove_except,
     save_artwork, tag_dir, update_from_tag)
