@@ -8,7 +8,8 @@ Contains objects used throughout puddletag"""
 from PyQt4.QtGui import *
 from PyQt4.QtCore import *
 from PyQt4.QtSvg import *
-import sys, os,pdb,shutil
+import json, sys, os,pdb,shutil
+from collections import defaultdict
 
 from itertools import groupby # for unique function.
 from bisect import bisect_left, insort_left # for unique function.
@@ -18,11 +19,11 @@ from audioinfo import (IMAGETYPES, DESCRIPTION, DATA, IMAGETYPE, DEFAULT_COVER,
     encode_fn, decode_fn, INFOTAGS)
 from operator import itemgetter
 path = os.path
-from configobj import ConfigObj
+from configobj import ConfigObj, ConfigObjError
 import traceback
 import time, re
 from glob import glob
-from constants import ACTIONDIR, SAVEDIR
+from constants import ACTIONDIR, SAVEDIR, CONFIGDIR
 from PyQt4.QtCore import QFile, QIODevice
 from StringIO import StringIO
 import itertools
@@ -206,63 +207,81 @@ class PuddleConfig(object):
     get -> load a key from a specified section
     set -> save a key section"""
     def __init__(self, filename = None):
-        self.settings = ConfigObj(filename, create_empty=True, encoding='utf8')
-
         if not filename:
-            filename = os.path.join(os.getenv('HOME'),'.puddletag', 'puddletag.conf')
+            filename = os.path.join(CONFIGDIR, 'puddletag.conf')
         self._setFilename(filename)
 
-        #TODO: backward compatibility, remove all.
         self.setSection = self.set
         self.load = self.get
 
     def get(self, section, key, default, getint = False):
-        settings = self.settings
+        settings = self.data
         try:
-            if isinstance(default, bool):
-                if self.settings[section][key] == 'True':
-                    return True
-                return False
-            elif getint or isinstance(default, (long,int)):
-                try:
-                    return int(self.settings[section][key])
-                except TypeError:
-                    return [int(z) for z in self.settings[section][key]]
-            else:
-                val = self.settings[section][key]
-                if val is None:
-                    return default
-                return val
+            value = self.data[section][key]
         except KeyError:
             return default
 
+        if isinstance(default, bool):
+            if value is True or value == 'True':
+                return True
+            return False
+        elif getint or isinstance(default, (long,int)):
+            try:
+                return int(value)
+            except TypeError:
+                return map(int, value)
+        else:
+            if value is None:
+                return default
+            return value
+
     def set(self, section = None, key = None, value = None):
-        settings = self.settings
-        if section in self.settings:
+        settings = self.data
+        if isinstance(value, QString):
+            value = unicode(value)
+        if section in self.data:
             settings[section][key] = value
         else:
             settings[section] = {}
             settings[section][key] = value
-        settings.write()
+        self.save()
+
+    def reload(self):
+        self.data = defaultdict(lambda: {})
+        if os.path.exists(self.filename):
+            try:
+                self.data.update(json.loads(open(self.filename, 'r').read()))
+            except:
+                pass
+
+    def save(self):
+        filename = self.filename
+        
+        if not os.path.exists(filename):
+            dirname = os.path.dirname(filename)
+            try:
+                os.makedirs(dirname)
+            except:
+                pass
+        
+        with open(filename, 'w') as fo:
+            fo.write(json.dumps(dict(self.data), indent=2))
 
     def _setFilename(self, filename):
-        dirname = os.path.dirname(filename)
-        if not os.path.exists(dirname):
-            os.makedirs(dirname)
-        self.settings.filename = filename
-        self.savedir = dirname
-        self.settings.reload()
+        self._filename = filename
+        self.savedir = os.path.dirname(filename)
+        self.reload()
 
     def _getFilename(self):
-        return self.settings.filename
+        return self._filename
 
     def sections(self):
-        return self.settings.keys()
+        return self.data.keys()
 
     filename = property(_getFilename, _setFilename)
 
 def _setupsaves(func):
-    filename = os.path.join(PuddleConfig().savedir, 'windowsizes')
+    filename = os.path.join(CONFIGDIR, 'windowsizes')
     settings = QSettings(filename, QSettings.IniFormat)
     return lambda x, y: func(x, y, settings)
 
@@ -1507,24 +1526,25 @@ class PicWidget(QWidget):
         self.lastfilename = u'~'
         self.currentFile = None
         self.filePattern = u'folder.jpg'
-        #The picture.
+
         self.label = ArtworkLabel()
         self.label.setFrameStyle(QFrame.Box)
-        #self.label.setMargin(10)
+
         self.label.setMinimumSize(200, 170)
         if buttons:
             self.label.setMaximumSize(200, 170)
         self._itags = []
-        #else:
-            #self.setMaximumSize(400,400)
+
         self.label.setAlignment(Qt.AlignCenter)
         self.connect(self.label, SIGNAL('newImages'), 
             lambda *filenames: self.addImages(self.loadPics(*filenames)))
-        #self.label.setSizePolicy(QSizePolicy.Fixed, QSizePolicy.Fixed)
 
-        #Description and picture type shit.
         self._image_desc = QLineEdit(self)
-        self._image_desc.setPlaceholderText(translate("Artwork", 'Enter a description'))
+        if (hasattr(self._image_desc, 'setPlaceholderText')):
+            self._image_desc.setPlaceholderText(translate("Artwork", 'Enter a description'))
+        else:
+            self._image_desc.setText('Enter a description')
+
         self._image_desc.setToolTip(
             translate("Artwork",
             '<p>Enter a description for the current cover.</p>'
@@ -2145,7 +2165,10 @@ class PuddleCombo(QWidget):
         values = [unicode(self.combo.itemText(index)) for index in xrange(self.combo.count())]
         values.append(unicode(self.combo.currentText()))
         cparser = PuddleConfig(self.filename)
-        cparser.setSection(self.name, 'values', values)
+        try:
+            cparser.setSection(self.name, 'values', values)
+        except ConfigObjError:
+            pass
 
     def removeCurrent(self):
         self.combo.removeItem(self.combo.currentIndex())
